/*
 * Copyright 2016 - 2023 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/model/model.hpp>

#include <keel/asset.hpp>
#include <keel/module.hpp>

#include "gfx.hpp"
#include "typeconv.hpp"

#include "module.hpp"

namespace nostalgia::core {

class CoreModule: public keel::Module {
	private:
		NostalgiaPaletteToPaletteConverter nostalgiaPaletteToPaletteConverter;
		TileSheetV1ToTileSheetConverter nostalgiaGraphicToTileSheetConverter;
		TileSheetToCompactTileSheetConverter tileSheetToCompactTileSheetConverter;
		TileSheetV2ToTileSheetConverter tileSheetV2ToTileSheetConverter;

	public:
		static CoreModule mod;
		[[nodiscard]]
		ox::Vector<keel::TypeDescGenerator> types() const noexcept override;
		[[nodiscard]]
		ox::Vector<const keel::BaseConverter*> converters() const noexcept override;
		[[nodiscard]]
		ox::Vector<keel::PackTransform> packTransforms() const noexcept override;
};

CoreModule CoreModule::mod;

const keel::Module *module() noexcept {
	return &CoreModule::mod;
}

ox::Vector<keel::TypeDescGenerator> CoreModule::types() const noexcept {
	return {
		keel::generateTypeDesc<TileSheetV1>,
		keel::generateTypeDesc<TileSheetV2>,
		keel::generateTypeDesc<TileSheet>,
		keel::generateTypeDesc<CompactTileSheet>,
		keel::generateTypeDesc<NostalgiaPalette>,
		keel::generateTypeDesc<Palette>,
	};
}

ox::Vector<const keel::BaseConverter*> CoreModule::converters() const noexcept {
	return {
		&nostalgiaPaletteToPaletteConverter,
		&nostalgiaGraphicToTileSheetConverter,
		&tileSheetToCompactTileSheetConverter,
		&tileSheetV2ToTileSheetConverter,
	};
}

ox::Vector<keel::PackTransform> CoreModule::packTransforms() const noexcept {
	return {
		// convert tilesheets to CompactTileSheets
		[](keel::Context *ctx, ox::Buffer *buff) -> ox::Error {
			oxRequire(hdr, keel::readAssetHeader(*buff));
			const auto typeId = ox::buildTypeId(hdr.clawHdr.typeName, hdr.clawHdr.typeVersion, hdr.clawHdr.typeParams);
			if (typeId == ox::buildTypeId<TileSheetV1>() ||
			    typeId == ox::buildTypeId<TileSheetV2>() ||
			    typeId == ox::buildTypeId<TileSheet>()) {
				oxReturnError(keel::convertBuffToBuff<core::CompactTileSheet>(
							ctx, *buff, ox::ClawFormat::Metal).moveTo(buff));
			}
			return {};
		},
	};
}

}
